function [ x_out,objective,iter_stop] = ADMM_Lasso( y,A,rho,lambda,e_abs,e_rel,maxIter)
%ADMM_LASSO, Johan Sw?rd, 29 Aug 2013
% ================================================================================================
%   [ x_out,objective,e_pri,e_dual,iter_stop] = ADMM_Lasso( y,A,rho,lambda,e_abs,e_rel,maxIter)
% ================================================================================================
% Solves ||y-Ax||_2^2+lambda||x||_1 using ADMM (1)
% Input:
%       y       - The data vector
%       A       - The dictionary
%       rho     - The augmented Lagrangian parameter
%       lambda  - The scalar regularization parameter
%       e_abs   - The absolute tolerance (Often set to 10^-3 or 10^-4
%       e_rel   - The relative tolerance (Often set to 10^-3 or 10^-4
%       maxIter - The number of maximum iterations
% Output:
%       x_out     - The estiamted x from (1)
%       objective - The resulting objective function (1) at each iteration
%       iter_stop - The last iteration


[N,K] = size(A); 

x = zeros(K,1);
z = zeros(K,1);
u = zeros(K,1);
counter=1;
e_pri(1) = sqrt(N)*e_abs+e_rel*max([norm(x),norm(-z),0]);
e_dual(1) = sqrt(K)*e_abs+e_rel*norm(rho*u);

objective(counter)=1/2*sum_square_abs(y-A*x)+lambda*norms(z,1);
qAi = inv(A'*A+rho*eye(K,K));

while maxIter>counter
    counter = counter + 1;
    zold = z;
    uold = u;
    
    x = qAi*(A'*y+rho*(zold-uold));
    z = SoftThresh(lambda/rho,x+uold);
    u = uold+x-z;

    r(counter) = norm(x-z);
    s(counter) = norm(rho*(-1)*((z-zold)));
    e_pri(counter) = sqrt(N)*e_abs+e_rel*max([norm(x),norm(-z),0]);
    e_dual(counter) = sqrt(K)*e_abs+e_rel*norm(rho*u);
    

    if e_pri(counter)>r(counter) && e_dual(counter)>s(counter)    
        break
    end
    objective(counter) = 1/2*sum_square_abs(y-A*x)+lambda*norms(z,1);
end
x_out = x(:,end);
iter_stop = counter-1;


end

function [ S ] = SoftThresh( kappa, alpha )
if alpha==0
    S=zeros(size(alpha));
else
    S = max(1-kappa./abs(alpha),0).*alpha;
end


end