% Written by: Maksim Butsenko, maksim.butsenko@gmail.com
% Paper: "ESTIMATING SPARSE SIGNALS USING INTEGRATED WIDE-BAND DICTIONARIES"
% https://arxiv.org/abs/1704.07584
% Authors: Maksim Butsenko, Johan Sw?rd, Andreas Jakobsson

% Simple example for 2-D data comparing narrowband and wideband
% dictionaries

N = 100; %nr of samples per dimension
t = (1:N)'; 
SNR = 10;
M = 7; % Number of bands per dimension
Q = 7; % zooming resource per active band
Mt = M*Q; % Number of intervals for finer grid
threshold = 0.1; %threshold for zooming step

%% finer grid dictionary
fBlock=[0:Mt-1]/(Mt);
A1 = zeros(N,Mt);
A2 = zeros(N,Mt);
for iM=1:Mt 
    a = exp(2i*pi*fBlock(iM)*t);
    A1(:,iM) = a/norm(a);
    a = exp(2i*pi*fBlock(iM)*t);
    A2(:,iM) = a/norm(a);
end;
A = kron(A2,A1);

%% banded dictionary
fBlock = linspace(0,.999,M+1);
A1 = zeros(N,M);
A2 = zeros(N,M);
for iM=1:M
    a = 1./(2i*pi*t).*(exp(2i*pi*t*(fBlock(iM+1)))-...
        exp(2i*pi*t*(fBlock(iM))));
    A1(:,iM) =a/norm(a);
    a = 1./(2i*pi*t).*(exp(2i*pi*t*(fBlock(iM+1)))-...
        exp(2i*pi*t*(fBlock(iM))));
    A2(:,iM) =a/norm(a);
end
Aband = kron(A2,A1);

%% construct signal and add noise
f1 = rand(1,2); %random freqs
f2 = rand(1,2);  %random freqs
Y = exp(2i*pi*f1(1)*t)*exp(2i*pi*f2(1)*t') +  exp(2i*pi*f1(2)*t)*exp(2i*pi*f2(2)*t');
y = Y(:);
yorg = y;
sigma = sqrt(norm(y)^2/length(y(:))*10^(-SNR/10));
noise = 1i*randn(N*N,1)+randn(N*N,1);
y = yorg+sigma*noise;

%% Reconstruct
% solve for narrowband dictionary
lambda=.4*max(abs(A'*y)); 
tic;
[x,~,~] = ADMM_Lasso(y,A,1,lambda,1e-6,1e-6,500);
time_n = toc;
% solve for wideband dictionary and zoom-in
lambda=.03*max(abs(Aband'*y));
tic;
[x_out,~,~] = ADMM_Lasso(y,Aband,1,lambda,1e-6,1e-6,500);
[x_zoom] = zoom_step_2D(x_out, threshold, Q, M, N, t, y, 0.25);
time_b = toc;

%% Plot solutions
az = 0;
el = 90;
figure;
% narrowband dictionary
fBlock=[0:Mt-1]/(Mt);
X = reshape(x,Mt,Mt);
subplot(2,2,1);
surf(fBlock,fBlock,abs(X./norm(X)));
shading interp;
hold on;
scatter3(f2, f1, [1; 1], 'r','filled'); %actual freqs
title('Narrowband Dictionary');

subplot(2,2,3); %top view
surf(fBlock,fBlock,abs(X));
shading interp;
xlabel('Normalized frequency');
ylabel('Normalized frequency');
view(az, el);
hold on;
h = scatter3(f2, f1, [40; 40], 'r','filled'); %actual freqs
legend(h,'True frequency');
str = ['Narrowband Dictionary' ', time to solve: ' num2str(time_n)];
title(str);


% wideband dictionary
fBlock = linspace(0,.999,M*Q + 1);
half_bin = fBlock(2)/2;
fBlock = fBlock(2:end) - half_bin; %place final solution in the middle of the band
subplot(2,2,2);
surf(fBlock,fBlock,abs(x_zoom./norm(x_zoom)));
shading interp;
hold on;
scatter3(f2, f1, [1; 1], 'r','filled'); %actual freqs
title('Wideband Dictionary');

subplot(2,2,4);%top view
surf(fBlock,fBlock,abs(x_zoom));
shading interp;
xlabel('Normalized frequency');
ylabel('Normalized frequency');
view(az, el);
hold on;
h = scatter3(f2, f1, [40; 40], 'r','filled'); %actual freqs
legend(h,'True frequency');
str = ['Wideband Dictionary' ', time to solve: ' num2str(time_b)];
title(str);
